import json
import requests
import time
import uuid
import wikipedia

class Card():
    def __init__(self, cardId=None, pageId=None, data=None):
        self.cardId = cardId if cardId else uuid.uuid4()
        delay = 2
        while True:
            try:
                self.pageId = pageId if pageId else int(
                    wikipedia.page(
                        wikipedia.random()
                        ).pageid
                    )
            except wikipedia.exceptions.PageError as e:
                # TODO: why is the random function returning lower-case pages?
                continue
            except wikipedia.exceptions.DisambiguationError as e:
                # TODO pick random disambiuation option
                continue
            except wikipedia.exceptions.WikipediaException as e:
                time.sleep(delay)
                delay = delay**2
                continue
            break
        if data:
            self.load_from_data(data)

    def validate(self):
        if not isinstance(self.cardId, uuid.UUID):
            raise TypeError(f"Card ID should be a UUID not {type(self.cardId).__name__}.")
        if not self.cardId.version == 4:
            raise self.Invalid(f"Card ID version should be 4 not {self.cardId.version}.")
        if not isinstance(self.pageId, int):
            raise TypeError(f"Page ID should be an integer not {type(self.pageId).__name__}.")
        try:
            wikipedia.page(pageid=self.pageId)
        # TODO: may need more precision here.
        except Exception as e:
            raise self.Invalid(f"Page ID {self.pageId} does not match a Wikipedia page.")

    def load_from_data(self, data):
        self.cardId = uuid.UUID(data['cardId'])
        self.pageId = data['pageId']

    def as_dict(self):
        return {
            "cardId": str(self.cardId),
            "pageId": self.pageId
            }

    def __str__(self):
        return json.dumps(self.as_dict(), sort_keys=True)

    class Invalid(Exception):
        def __init__(self, message, statusCode=406):
            super().__init__(message)
            self.message = message
            self.statusCode = statusCode
