class Database():
    SQL_GET_BUY_ORDERS = """
        SELECT *
        FROM orders
        WHERE orderType == 'buy';
        """

    SQL_GET_UNFULFILLED_BUY_ORDERS = """
        SELECT b.buyOrderId, b.price, b.volume - COUNT(t.sqlId) AS remainingVolume
        FROM buy_orders b
        JOIN transactions t
        ON b.buyOrderId = t.buyOrderId
        WHERE b.expires > CURRENT_TIMESTAMP()
        GROUP BY b.buyOrderId
        HAVING COUNT(t.sqlId) < b.volume
        ORDER BY b.timestamp ASC
        """

    SQL_GET_AVAILABLE_SELL_ORDER_ITEMS_FOR_BUY_ORDER = """
        SELECT o.sellOrderId, o.price, o.fee, o.userId, i.sqlId, i.cardId
        FROM sell_orders o
        JOIN sell_order_items i
        ON o.sellOrderId = i.sellOrderId
        WHERE i.cardId == {}
        AND o.price <= {}
        AND o.expires > CURRENT_TIMESTAMP()
        AND i.sqlId NOT IN (
            SELECT sellOrderItemSqlId
            FROM transactions
            WHERE isAbandoned == false;
        )
        ORDER BY price DESC;
        """

    SQL_INSERT_TRANSACTION = """
        INSERT INTO transactions (transactionId, buyOrderId, sellOrderItemSqlId)
        VALUES ({}, {}, {});
        """

    def __init__(self):
        self.conn = mariadb.connect(
            # TODO: default values
            host = os.getenv("ORDER_MATCHER_DB_HOST", None),
            port = os.getenv("ORDER_MATCHER_DB_PORT", 3342),
            user = os.getenv("ORDER_MATCHER_DB_USER", None),
            password = os.getenv("ORDER_MATCHER_DB_PASSWORD", None)
            )

    def get_available_sell_order_items_for_buy_order(self, buyOrderId, price):
        return self.conn.cursor().execute(
            SQL_GET_AVAILABLE_SELL_ORDER_ITEMS_FOR_BUY_ORDER.format(
                buyOrderId,
                price
                )
            )

    def insert_transaction(self, transactionId, buyOrderId, sellOrderItemSqlId):
        self.conn.cursor().execute(
            SQL_INSERT_TRANSACTION.format(
                    transactionId,
                    buyOrderId,
                    sellOrderItemSqlId
                )
            )
